''
'' This code is part of https://developer.mescius.com/document-solutions/dot-net-pdf-api/demos.
'' Copyright (c) MESCIUS inc. All rights reserved.
''
Imports System
Imports System.Collections.Generic
Imports System.Text
Imports System.Text.RegularExpressions
Imports System.Threading
Imports System.IO
Imports System.Linq
Imports System.Drawing
Imports System.Runtime.CompilerServices
Imports System.Runtime.InteropServices
Imports GrapeCity.Documents.Drawing
Imports GrapeCity.Documents.Text
Imports GrapeCity.Documents.Pdf
Imports GrapeCity.Documents.Pdf.Annotations
Imports GrapeCity.Documents.Pdf.Actions
Imports GrapeCity.Documents.Html
Imports GCTEXT = GrapeCity.Documents.Text
Imports GCDRAW = GrapeCity.Documents.Drawing

Public Module Util

    Public Const LargeDocumentIterations = 1000
    Private s_gen As Integer = 0
    Private s_fixedTime As DateTime = New DateTime(1989, 6, 21, 1, 23, 45, DateTimeKind.Utc)

    Public Property NoRandom As Boolean = False

    Public Function NewRandom() As Random
        If NoRandom Then
            Return New Random(0)
        Else
            Return New Random(Convert.ToInt32(DateTime.Now.Ticks And Int32.MaxValue) + Interlocked.Increment(s_gen))
        End If
    End Function

    Public Function TimeNow() As DateTime
        If NoRandom Then
            Return s_fixedTime
        Else
            Return DateTime.Now
        End If
    End Function

    Private s_words As String() =
        {
            "lorem", "ipsum", "dolor", "sit", "amet", "consectetur", "adipiscing", "elit", "mauris", "id", "volutpat",
            "tellus", "ullamcorper", "sem", "praesent", "et", "ante", "laoreet", "lobortis", "nunc", "congue",
            "nisi", "donec", "ac", "tempus", "sed", "feugiat", "pulvinar", "pharetra", "turpis", "nonummy",
            "at", "felis", "eget", "molestie", "euismod", "non", "aliquet", "diam", "proin", "mi", "nibh",
            "massa", "tincidunt", "ut", "dolore", "magna", "aliquam", "erat"
        }

    Private s_browserBasePath As String = Nothing
    Private ReadOnly s_browserBasePath_lock As New Object()

    '' Set this to use the browser from a known location
    Public Sub SetBrowserBasePath(ByVal browserBasePath As String)
        SyncLock s_browserBasePath_lock
            s_browserBasePath = browserBasePath
        End SyncLock
    End Sub

    '' DsHtml v6.0+: this method provides a single target location to be used
    '' by the online samples, and also by all samples downloaded by a user,
    '' so that multiple samples can share the same browser installation
    '' rather than each sample downloading and installing its own copy.
    '' On a system with Chrome installed, the BrowserFetcher.GetSystemChromePath()
    '' call ensures that the installed browser is used.
    Public Function GetBrowserPath() As String
        If s_browserBasePath Is Nothing Then
            Dim systemPath = BrowserFetcher.GetSystemChromePath()
            If systemPath IsNot Nothing Then
                Return systemPath
            End If
        End If
        Dim pth As String
        If s_browserBasePath Is Nothing Then
            pth = Path.Combine(Path.GetTempPath(), ".gc-chromium")
        Else
            pth = s_browserBasePath
        End If
        Dim bf = New BrowserFetcher() With {.DestinationFolder = pth}
        Return bf.GetDownloadedPath()
    End Function

    '' DsHtml v6.0
    Public Function NewHtmlBrowser() As GcHtmlBrowser
        Dim pth = GetBrowserPath()
        Try
            If (RuntimeInformation.IsOSPlatform(OSPlatform.Linux)) Then
                Return New GcHtmlBrowser(pth, New LaunchOptions With {.RunWithNoSandbox = True})
            Else
                Return New GcHtmlBrowser(pth)
            End If
        Catch ex As Exception
            If pth = BrowserFetcher.GetSystemChromePath() Then
                Throw
            Else
                Throw New Exception("Exception occurred when launching the browser (see inner exception for details), " +
                    $"the browser installation in {Path.GetDirectoryName(pth)} may be corrupt. Please remove it and try again.", ex)
            End If
        End Try
    End Function

    '' https://stackoverflow.com/questions/4286487/is-there-any-lorem-ipsum-generator-in-c
    Public Function LoremIpsum(
            Optional ByVal numParagraphs As Integer = 5,
            Optional ByVal minSentences As Integer = 5,
            Optional ByVal maxSentences As Integer = 10,
            Optional ByVal minWords As Integer = 5,
            Optional ByVal maxWords As Integer = 30) As String

        Dim commaMax = 10
        Dim rnd = NewRandom()
        Dim numSentences = rnd.Next(maxSentences - minSentences) + minSentences + 1
        Dim numWords = rnd.Next(maxWords - minWords) + minWords + 1

        Dim result = New StringBuilder()
        For p = 1 To numParagraphs
            For s = 1 To numSentences
                For w = 1 To numWords
                    Dim word = s_words(rnd.Next(s_words.Length))
                    If w = 1 Then
                        result.Append(Char.ToUpper(word(0)))
                        result.Append(word.Substring(1))
                    Else
                        If rnd.Next(commaMax) = 0 Then
                            result.Append(",")
                        End If
                        result.Append(" ")
                        result.Append(word)
                    End If
                Next
                result.Append(". ")
            Next
            result.Append(vbCrLf)
        Next
        Return result.ToString()
    End Function

    '' Gets the list of words used by the LoremIpsum method.
    Public Function LoremWords() As List(Of String)
        Return s_words.ToList()
    End Function

    '' https://stackoverflow.com/questions/273313/randomize-a-listt
    <Extension()>
    Public Sub Shuffle(Of t)(ByVal list As IList(Of t))
        Dim rnd = NewRandom()
        Dim n = list.Count
        While (n > 1)
            n -= 1
            Dim k = rnd.Next(n + 1)
            Dim value = list(k)
            list(k) = list(n)
            list(n) = value
        End While
    End Sub

    ''' <summary>
    ''' Adds a note to the document - a text fragment on a colored background.
    ''' Used to insert meta info into sample documents.
    ''' </summary>
    ''' <param name="text">The text to draw.</param>
    ''' <param name="page">The page to draw on.</param>
    ''' <param name="bounds">The text bounds. If null, whole page with 1" margins all around is used.</param>
    ''' <returns>The actual rectangle occupied by the note.</returns>
    Public Function AddNote(ByVal text As String, ByVal page As Page, Optional ByVal bounds As RectangleF? = Nothing) As RectangleF
        Dim g = page.Graphics
        Dim tl = g.CreateTextLayout()
        Dim pad = g.Resolution / 8.0F
        tl.DefaultFormat.Font = StandardFonts.Helvetica
        Dim yumin = Path.Combine("Resources", "Fonts", "yumin.ttf")
        If (File.Exists(yumin)) Then
            tl.DefaultFormat.Font.AddLinkedFont(GCTEXT.Font.FromFile(yumin))
        End If
        tl.DefaultFormat.FontSize = 12
        If Not bounds.HasValue Then
            Dim margin = g.Resolution ' 1" margins by default
            bounds = New RectangleF(margin, margin, page.Size.Width - margin * 2, page.Size.Height - margin * 2)
        ElseIf bounds.Value.Height = 0 Then
            bounds = New RectangleF(bounds.Value.Location, New SizeF(bounds.Value.Width, page.Size.Height - bounds.Value.Top * 2))
        End If
        tl.MaxWidth = bounds.Value.Width
        tl.MaxHeight = bounds.Value.Height
        tl.MarginAll = pad

        Dim texts = Regex.Split(text, "(https?://[^ ]+)", RegexOptions.IgnoreCase)
        Dim runs = New List(Of TextRun)()
        Dim urls = New List(Of String)()

        For Each txt In texts
            If (txt.StartsWith("http", StringComparison.OrdinalIgnoreCase)) Then
                urls.Add(txt)
                runs.Add(tl.Append(txt, New TextFormat(tl.DefaultFormat) With {.ForeColor = Color.Blue, .Underline = True, .TextRunAsCluster = True}))
            Else
                tl.Append(txt)
            End If
        Next
        tl.PerformLayout(True)
        For i = 0 To runs.Count - 1
            Dim run = runs(i)
            Dim rects = tl.GetTextRects(run.CodePointIndex, run.CodePointCount)
            For Each r In rects
                Dim rc = r.ToRectangleF()
                rc.Offset(bounds.Value.Location)
                page.Annotations.Add(New LinkAnnotation(rc, New ActionURI(urls(i))))
            Next
        Next

        Dim rect = tl.ContentRectangle
        rect.Offset(bounds.Value.Location)
        rect.Inflate(pad, pad)

        g.FillRectangle(rect, Color.FromArgb(213, 240, 221))
        g.DrawRectangle(rect, Color.FromArgb(59, 170, 92), 0.5F)
        g.DrawTextLayout(tl, bounds.Value.Location)

        Return rect
    End Function

    ''' <summary>
    ''' Loads an image from a disk file.
    ''' <para>
    ''' The only point of this method Is to provide better performance on a Windows system,
    ''' by using GrapeCity.Documents.Drawing.Windows.WicImage which works via the
    ''' Windows Imaging Component framework. If WicImage Is Not supported, this method
    ''' defaults to platform-independent Image.FromFile() method.
    ''' </para>
    ''' </summary>
    ''' <param name="path">The image path.</param>
    ''' <returns>The Image object.</returns>
    Public Function ImageFromFile(ByVal path As String) As IImage
        If (GrapeCity.Documents.Imaging.Windows.GcWicBitmap.IsSupported) Then
            Return New GrapeCity.Documents.Imaging.Windows.GcWicBitmap(path)
        Else
            Return GCDRAW.Image.FromFile(path)
        End If
    End Function
End Module
